/*
  the yaf producer interface for the deviceConfigServer/streamProducer
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#ifndef __YAFPRODUCER_H
#define __YAFPRODUCER_H

#include <producer/core/producer.h>
#include <deviceConfig/deviceConfig.h>
#include <util/controlChannel.h>
#include <yafxplayer/xplayerRuntime.defs>
#include <producer/yaf/yafXPlayerControl.h>
#include <producer/yaf/yafStream.h>

#include <signal/toolkit/notifyMainThread.h>
#include <pthread.h>



/**
  
  Note:<p>
  StreamProducer are Device which creates a thread they communicate
  with a deviceConfigServer and the server coordinated the
  transitions from the old to the new deviceConfig Entries.
  <p>
  This class is the class with which the server communicates!
  It is not a "streamProducer".
  <p>

  This class implements the abstract methods for the producer.
  You can use this class in a streamProducer then it creates
  new deviceConfig classes. (especially : PCM audio)
  <p>
  The yaf interface is a bit complex because it must handle
  a command line yaf-player.
  <p>
*/


class YafProducer :  public NotifyMainThreadClient, 
		     public Producer,
		            ControlChannel {
  DeviceConfig* config;
  MusicInfo* musicInfo;
  YafAudioInfo* yafAudioInfo;
  StatusInfo* statusInfo;
  AudioBuffer* audioBuffer;

  YafXPlayerControl* control;
  struct YafStream* stream;
  int instanceCnt;

 public:
  YafProducer();
  ~YafProducer();

  void updateDeviceConfig(DeviceConfig* newConfig);

  int open(char* filename );
  int close();
  int play();
  int pause();
  int jump(int);


  // Not a part of the Producer interface.
  // These things are special for the yafInterface:

  void processRuntimeCommand(int command,char* args);
  void processReturnCommand(int cmdNr,int cmdId,char* ret,char* args);

  void addArgument(char* arg);
  void startDecoder();

  void bufferFilledNotify(int filled);  // called by yafStream after underrun
  void setThreadState(int state);

  void processNotify(char msgId);        // called when thread "send signal"
                                        // but _is called_ by main-thread(gui)

 private:
  void clearStream();
  void dataUpdate();

  int lClearRequest;
  int pauseCmdNr;
  int fullUpdate;

  long eofBytes;
  NotifyMainThread* notifyBridge;

};

#endif
