/*
  a scaler class. (I dont know what this really mean)
  Copyright (C) 1998  Martin Vogt;Philip VanBaren, 2 September 1993

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#include <util/scaler.h>



Scaler::Scaler(int points) {
  this->points=points;

  output=(fft_type *)malloc(points*sizeof(fft_type));
  ybase=(long *)malloc(MAX_LEN/2*sizeof(long));



  // set values for ybase
  setup_logscales();


}


Scaler::~Scaler() {
  delete output;
}






/*
 *  Set up logarithmic amplitude scale factors and offsets.
 */
void Scaler::setup_logscales()
{
   int i;
   double scale,base,convert,offset;
   int log_base=6; // Flag set for base of log scale (default=6 = -60db) 
   int logs=0;     // Flag for max of log scale (default=0 = 0db) 
   int deriv=0;    // Flag for doing differencing for 6db/octave gain 
   long ref_freq=1000;     // Reference frequency for n db/octave gains 
   int shift=0;            // Number of bits for gain shift 
   int gain3db=0;          // Flag indicating a 3db/octave scale factor gain 
   long SampleRate=44100L; // A/D sampling rate 
   float disp_scalefactor; // Display scalefactor for log values 
   

   /*
    *  Compute the (logarithmic) y scale factor and offset.
    *  This may include a 3dB/octave gain.
    *
    *  Conversion factor from db/10 to dPhils (the computed "unit")
    *  where a factor of 2 yields 16384 dPhils (6.02dB)
    *  Scaling factor is such that  32768 ->   0.00 dB -> 245760 dPhils
    *                          and      2 -> -84.29 dB ->  16384 dPhils
    *                          and      1 -> -90.31 dB ->      0 dPhils
    *  i.e. dPhils=16384.0/log(2) * log(value)
    * and changes of 6.02 dB = 16384 dPhils
    */
   convert=819.2*log(10)/log(2);  /* Scale for dB to dPhils conversion */
   offset=log10(32768L)*20;       /* Offset for db to dPhils conversion */

   /*
    * This value is used in the main program group to convert squared values
    * amplitudes to dPhils using dPhils = log(value^2)*log_scalefactor
    */

   scale=(WINDOW_BOTTOM-WINDOW_TOP)/(10*(log_base-logs)*convert);
   if(deriv==0)
      base=(offset-log_base*10)*convert;
   else if(deriv==1)
      base=(offset-log10(SampleRate/(2*M_PI*ref_freq))*20-log_base*10)*convert;
   else
      base=(offset-log10(SampleRate/(2*M_PI*ref_freq))*40-log_base*10)*convert;

   disp_scalefactor=scale; // Save the unshifted version for avg. display mode

   shift=0;
   /*
    *  Make maximum use of available bits
    *  (use only 12 bits--other 4 used for higher resolution in the data)
    */
   while(scale<4096)
   {
      scale*=2;
      shift++;
   }
   scale=floor(scale+0.5);

   for(i=0;i<(WINDOW_RIGHT-WINDOW_LEFT+1);i++)
      yscale[i]=(unsigned int)scale;

   if(gain3db)
   {
      ybase[0]=500000L;
      for(i=1;i<points/2;i++)
	 ybase[i]=(long)floor(0.5+base - log10((double)i*(double)SampleRate
                        /(double)points/(double)ref_freq)*convert*10);
   }
   else
   {
     for(i=0;i<points/2;i++) {
       ybase[i]=(long)floor(0.5+base);
       //       cout << "yabase["<<i<<"]="<<ybase[i]<<endl;
     }
      
   }
     //   shiftscale=1/pow(2,shift);
}





void Scaler::processScale(fft_type* data,int* bitReversed) {
  int i;

  fft_type a2;

  for(i=0;i<points/2;i++)
    {
      // Compute the log(squared magnitude) 
      long re=(long)data[bitReversed[i]];
      long im=(long)data[bitReversed[i+1]];
      double root=32768L;
      a2=re*re+im*im;
      if (a2<0) a2=0;  // Watch for possible overflow 
      root=sqrt(a2);
      output[i]=root;
    }
}


double* Scaler::getScaleArray() {
  return output;
}






